"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_languageserver_1 = require("vscode-languageserver");
const browser_1 = require("vscode-languageserver/browser");
const vscode_languageserver_textdocument_1 = require("vscode-languageserver-textdocument");
const built_in_functions_1 = require("./assets/built-in-functions");
const constant_1 = require("./constant");
const execute_commands_1 = require("./execute-commands");
const lsp_server_1 = require("./lsp-server");
const messageReader = new browser_1.BrowserMessageReader(self);
const messageWriter = new browser_1.BrowserMessageWriter(self);
const connection = (0, browser_1.createConnection)(messageReader, messageWriter);
connection.onInitialize(() => {
    const capabilities = {
        textDocumentSync: vscode_languageserver_1.TextDocumentSyncKind.Incremental,
        completionProvider: {
            triggerCharacters: '.1234567890abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'.split(''),
            resolveProvider: true
        },
        executeCommandProvider: {
            commands: [constant_1.VALIDATION_ON_COMMAND, constant_1.REGISTER_SCHEMAS_COMMAND]
        },
        documentFormattingProvider: true,
        hoverProvider: true,
        foldingRangeProvider: true
    };
    return { capabilities };
});
const documents = new vscode_languageserver_1.TextDocuments(vscode_languageserver_textdocument_1.TextDocument);
const lspServer = new lsp_server_1.LSPServer();
let validationOn = false;
documents.listen(connection);
documents.onDidClose(e => {
    lspServer.warehouse.removeParsingValue(e.document.uri);
});
documents.onDidChangeContent(change => {
    if (!lspServer.warehouse.hasParsingValue(change.document)) {
        lspServer.warehouse.setParsingValue(change.document);
    }
    else {
        const parseValueCache = lspServer.warehouse.getParsingValue(change.document);
        if (change.document.getText() === parseValueCache.origin) {
            return;
        }
        lspServer.warehouse.setParsingValue(change.document);
    }
    if (validationOn) {
        validateTextDocument(change.document);
    }
});
function validateTextDocument(textDocument) {
    const diagnostics = lspServer.doValidation(textDocument);
    connection.sendDiagnostics({ uri: textDocument.uri, diagnostics }).then();
}
connection.onCompletion((params) => {
    const { textDocument, position } = params;
    const document = documents.get(textDocument.uri);
    if (!document) {
        console.error('Language Server: TextDocument not found.');
        return [];
    }
    return lspServer.doCompletion(document, position);
});
connection.onCompletionResolve((item) => {
    if (!item.detail) {
        item.detail = 'keyword';
    }
    return item;
});
connection.onRequest(vscode_languageserver_1.DocumentDiagnosticRequest.type, params => {
    const textDocument = documents.get(params.textDocument.uri);
    if (!textDocument) {
        console.error('Language Server: TextDocument not found.');
        return { kind: 'full', items: [] };
    }
    const items = lspServer.doValidation(textDocument);
    return { kind: 'full', items };
});
connection.onExecuteCommand(params => {
    if (params.command === constant_1.VALIDATION_ON_COMMAND) {
        validationOn = !!params.arguments?.[0];
        if (!validationOn) {
            documents.all().forEach(d => {
                connection.sendDiagnostics({ uri: d.uri, diagnostics: [] }).then();
            });
        }
    }
    else {
        (0, execute_commands_1.handleExecuteCommand)(documents, lspServer, params);
    }
});
connection.onDocumentFormatting((params) => {
    const document = documents.get(params.textDocument.uri);
    if (!document) {
        console.error('Language Server: TextDocument not found.');
        return [];
    }
    return lspServer.doFormatting(document, params.options);
});
connection.onHover((params) => {
    const document = documents.get(params.textDocument.uri);
    const contents = [];
    if (!document) {
        console.error('Language Server: TextDocument not found.');
        return { contents };
    }
    const token = lspServer.getTokenByPosition(document, params.position);
    if (token?.text === undefined) {
        return { contents };
    }
    const functionItem = built_in_functions_1.BuiltInFunctions.find(f => token.text.localeCompare(f.expr.replace('()', ''), undefined, { sensitivity: 'accent' }) === 0);
    if (functionItem) {
        contents.push(`**System (Built-in) Function**`);
        contents.push(functionItem.description.value);
        contents.push(`For more information, please refer to [Spark Document](https://spark.apache.org/docs/latest/sql-ref-functions-builtin.html).`);
    }
    return {
        contents
    };
});
connection.onFoldingRanges((params) => {
    const document = documents.get(params.textDocument.uri);
    if (!document) {
        console.error('Language Server: TextDocument not found.');
        return [];
    }
    return lspServer.doFoldingRanges(document);
});
connection.listen();
//# sourceMappingURL=server-worker.js.map